package vsp;

import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimMethodNotSupportedException;
import cz.zcu.fav.kiv.jsim.JSimSecurityException;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimSimulationAlreadyTerminatedException;
import cz.zcu.fav.kiv.jsim.JSimTooManyHeadsException;
import cz.zcu.fav.kiv.jsim.JSimTooManyProcessesException;

/**
 * 
 * VSP
 * 4/3
 * 
 * @author Jan Tichava <jtichava@students.zcu.cz>
 *
 */
public class Vsp {
	
	/**
	 * Defined exponential distribution ratio
	 */
	private static final double lambda = 0.33;
	/**
	 * Defined mean value for server 1
	 */
	private static final double ts1 = 1.5;
	/**
	 * Defined mean value for server 2
	 */
	private static final double ts2 = 2;
	/**
	 * Defined variation of the mean value for server 1
	 */
	private static final double cs1 = 0.5;
	/**
	 * Defined variation of the mean value for server 2
	 */
	private static final double cs2 = 0.5;
	/**
	 * Probability for branch 1
	 */
	private static final double p1 = 0.8;
	/**
	 * Probability for branch 2
	 */
	private static final double p2 = 1 - p1;
	
	private static final int DEFAULT_STEPS = 10000;
	
	/**
	 * Main simulation
	 * @param args command line arguments
	 */
	public Vsp(String[] args) {
		int steps = parseArguments(args);
		
		JSimSimulation simulation = null;
		
		try {
			
			System.out.println("Initializing the simulation..");
			
			simulation = new JSimSimulation("Queueing Networks Simulation");
			
			QueueWithServer queue1 = new QueueWithServer("Queue 1", simulation, p1);
			QueueWithServer queue2 = new QueueWithServer("Queue 2", simulation, p2);
			
			Server server1 = new Server("Server 1", simulation, ts1, cs1, queue1);
			Server server2 = new Server("Server 2", simulation, ts2, cs2, queue2);
			
			Generator generator = new Generator("Generator", simulation, lambda, queue1, queue2);
			
			queue1.setServer(server1);
			queue2.setServer(server2);
			
			simulation.message("Activating the generators...");
			
			generator.activate(0);
			
			simulation.message("Running the simulation, please wait.");
			
			int i = 0;
			while ((i < steps) && (simulation.step() == true)) {
				i++;
			}
			
			simulation.message("Simulation interrupted at time " + simulation.getCurrentTime());
			simulation.message("Generated transactions: " + generator.getGenerated());
			simulation.message("Queues statistics:");
			simulation.message(queue1.getHeadName() + ": Lw = " + queue1.getLw());
			simulation.message(queue2.getHeadName() + ": Lw = " + queue2.getLw());
			simulation.message("Servers statistics:");
			simulation.message(server1.getName() + ": Tq = " + server1.getTq());
			simulation.message(server2.getName() + ": Tq = " + server2.getTq());
			simulation.message("Global statistics:");
			simulation.message("Lq = " + generator.getLq());
			simulation.message("Tq = " + ((server1.getTransTq() + server2.getTransTq()) / (server1.getCounter() + server2.getCounter())));
			
		} catch (JSimInvalidParametersException e) {
			e.printStackTrace();
		} catch (JSimTooManyHeadsException e) {
			e.printStackTrace();
		} catch (JSimSimulationAlreadyTerminatedException e) {
			e.printStackTrace();
		} catch (JSimTooManyProcessesException e) {
			e.printStackTrace();
		} catch (JSimSecurityException e) {
			e.printStackTrace();
		} catch (JSimMethodNotSupportedException e) {
			e.printStackTrace();
		} finally {
			simulation.shutdown();
		}	
	}
	
	/**
	 * Parse command line arguments
	 * @param args command line argument
	 * @return number of steps
	 */
	private int parseArguments(String[] args) {
		int steps = 0;
		
		try {
			if (args.length != 1) {
				printHelp();
				
				steps = DEFAULT_STEPS;
			}
			else {
				steps = Integer.valueOf(args[0]);
			}
		} catch (NumberFormatException e) {
			System.err.println("Error: " + e.getMessage());
			printHelp();
			System.exit(0);
		}
		
		return steps;
	}
	
	/**
	 * Print help
	 */
	private void printHelp() {
		System.out.println("Help:");
		System.out.println("java -jar Vsp.jar count");
	}
	
	/**
	 * Create main object
	 * @param args command line arguments
	 */
	public static void main(String[] args) {
		new Vsp(args);
	}

}
